﻿using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Drawing;
using System.Linq;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Design;
using System.Workflow.ComponentModel.Compiler;
using System.Workflow.ComponentModel.Serialization;
using System.Workflow.Runtime;
using System.Workflow.Activities;
using System.Workflow.Activities.Rules;
using System.Net;
using System.IO;
using System.Xml;
using System.ServiceModel.Syndication;
using System.Collections.Generic;

namespace BlogMonitor
{
    public partial class BlogMonitorActivity : System.Workflow.ComponentModel.Activity
    {

        public string FeedUrl
        {
            get { return (string)GetValue(FeedUrlProperty); }
            set { SetValue(FeedUrlProperty, value); }
        }

        public static readonly DependencyProperty FeedUrlProperty =
            DependencyProperty.Register("FeedUrl", typeof(string), typeof(BlogMonitorActivity));

        public string SummaryKeyword
        {
            get { return (string)GetValue(SummaryKeywordProperty); }
            set { SetValue(SummaryKeywordProperty, value); }
        }

        public static readonly DependencyProperty SummaryKeywordProperty =
            DependencyProperty.Register("SummaryKeyword", typeof(string), typeof(BlogMonitorActivity));


        public List<SyndicationItem> FeedItems
        {
            get { return (List<SyndicationItem>)GetValue(FeedItemsProperty); }
            set { SetValue(FeedItemsProperty, value); }
        }

        public static readonly DependencyProperty FeedItemsProperty =
            DependencyProperty.Register("FeedItems", typeof(List<SyndicationItem>), typeof(BlogMonitorActivity));


        public string FeedTitle
        {
            get { return (string)GetValue(FeedTitleProperty); }
            set { SetValue(FeedTitleProperty, value); }
        }

        public static readonly DependencyProperty FeedTitleProperty =
            DependencyProperty.Register("FeedTitle", typeof(string), typeof(BlogMonitorActivity));



        public BlogMonitorActivity()
        {
            InitializeComponent();
        }

        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            SyndicationFeed feed = GetFeed(FeedUrl);

            FeedTitle = feed.Title.Text;

            FeedItems = (from item in feed.Items
                            where (SummaryKeyword.Length > 0 ? item.Summary.Text.Contains(SummaryKeyword) : true)
                            orderby item.LastUpdatedTime descending, item.PublishDate descending
                            select item).ToList() ;

            return ActivityExecutionStatus.Closed;
        }

        protected SyndicationFeed GetFeed(string feedUri)
        {
            WebClient webClient = new WebClient();
            string feedContent = webClient.DownloadString(feedUri);

            StringReader stringReader = new StringReader(feedContent);
            XmlTextReader xmlTextReader = new XmlTextReader(stringReader);

            SyndicationFeed feed = SyndicationFeed.Load(xmlTextReader);

            return feed;
        }
    }
}
