﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel.Syndication;
using System.Runtime.Serialization;
using System.Xml;
using System.IO;

namespace BlogMonitor
{
    [Serializable]
	public class FeedResult: ISerializable
	{
        public string FeedTitle { get; set; }
        public DateTime LastPollTime { get; set; }
        public List<SyndicationItem> FeedItems { get; set; }

        public FeedResult()
        {
        }

        protected FeedResult(SerializationInfo info, StreamingContext context)
        {
            FeedTitle = info.GetString("FeedTitle");
            LastPollTime = info.GetDateTime("LastPollTime");

            List<string> feedItemsXml = info.GetValue("FeedItems", typeof(List<string>)) as List<string>;
            FeedItems = new List<SyndicationItem>();
            foreach (string itemXml in feedItemsXml)
            {
                SyndicationItem item = SyndicationItem.Load(XmlReader.Create(new StringReader(itemXml)));
                FeedItems.Add(item);
            }
        }

        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("FeedTitle", FeedTitle);
            info.AddValue("LastPollTime", LastPollTime);

            List<string> feedItemsXml = new List<string>();

            StringBuilder sb;
            foreach (SyndicationItem item in FeedItems)
            {
                sb = new StringBuilder();
                XmlWriter writer = XmlWriter.Create(sb);
                item.GetAtom10Formatter().WriteTo(writer);
                writer.Flush();
                writer.Close();
                feedItemsXml.Add(sb.ToString());
            }

            info.AddValue("FeedItems", feedItemsXml);
        }
    }
}
