﻿using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Drawing;
using System.Linq;
using System.Workflow.ComponentModel.Compiler;
using System.Workflow.ComponentModel.Serialization;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Design;
using System.Workflow.Runtime;
using System.Workflow.Activities;
using System.Workflow.Activities.Rules;
using System.Collections.Generic;
using BlogMonitor;
using WorkflowSimulator;

namespace BlogMonitorSequentialWorkflow
{
    [WorkflowSimulationUI(LocalServiceType = "BlogMonitor.BlogMonitor, BlogMonitorLocalService", UserInterfaceType = "BlogMonitorUI.BlogMonitorPanel, BlogMonitorUI")]
    public sealed partial class BlogMonitorSequential : SequentialWorkflowActivity
    {
        public List<FeedQuery> FeedsToMonitor { get; set; }

        public TimeSpan CollectAt { get; set; }

        public TimeSpan SendAt { get; set; }

        public Dictionary<string, FeedResult> MonitoringResults { get; set; }

        public bool MonitoringCanceled { get; set; }

        public BlogMonitorSequential()
        {
            InitializeComponent();

            MonitoringResults = new Dictionary<string, FeedResult>();
        }

        private void ComputeCollectAtDelay(object sender, EventArgs e)
        {
            TimeSpan delay = ComputeDelay(CollectAt);
            DelayActivity delayActivity = sender as DelayActivity;
            delayActivity.TimeoutDuration = delay;
        }

        private void ComputeSendAtDelay(object sender, EventArgs e)
        {
            TimeSpan delay = ComputeDelay(SendAt);
            DelayActivity delayActivity = sender as DelayActivity;
            delayActivity.TimeoutDuration = delay;
        }

        private TimeSpan ComputeDelay(TimeSpan targetTimeOfDay)
        {
            TimeSpan delay;
            TimeSpan currentTimeOfDay = DateTime.Now.TimeOfDay;

            if (currentTimeOfDay.TotalSeconds > targetTimeOfDay.TotalSeconds)
            {
                //schedule for right away
                delay = new TimeSpan(0);
            }
            else
            {
                //schedule for same day
                delay = targetTimeOfDay - currentTimeOfDay;
            }

            return delay;
        }

        private void InitMonitorActivity(object sender, ReplicatorChildEventArgs e)
        {
            BlogMonitorActivity blogMonitorActivity = e.Activity as BlogMonitorActivity;
            FeedQuery feedQuery = e.InstanceData as FeedQuery;
            blogMonitorActivity.FeedUrl = feedQuery.FeedUrl;
            blogMonitorActivity.SummaryKeyword = feedQuery.SearchKeyword;
        }

        private void CompleteMonitorActivity(object sender, ReplicatorChildEventArgs e)
        {
            BlogMonitorActivity blogMonitorActivity = e.Activity as BlogMonitorActivity;

            if (MonitoringResults.ContainsKey(blogMonitorActivity.FeedUrl))
            {
                MonitoringResults[blogMonitorActivity.FeedUrl] = new FeedResult()
                {
                    FeedTitle = blogMonitorActivity.FeedTitle,
                    FeedItems = blogMonitorActivity.FeedItems,
                    LastPollTime = DateTime.Now
                };
            }
            else
            {
                MonitoringResults.Add(blogMonitorActivity.FeedUrl,
                                        new FeedResult()
                                        {
                                            FeedTitle = blogMonitorActivity.FeedTitle,
                                            FeedItems = blogMonitorActivity.FeedItems,
                                            LastPollTime = DateTime.Now
                                        });
            }
        }

        private void UpdateSendAt(object sender, EventArgs e)
        {
            SendAt = SendAt.Add(TimeSpan.FromDays(1.0));
        }

        private void UpdateCollectAt(object sender, EventArgs e)
        {
            CollectAt = CollectAt.Add(TimeSpan.FromDays(1.0));
        }

        private void SetMonitoringCanceled(object sender, ExternalDataEventArgs e)
        {
            this.MonitoringCanceled = true;
        }

        private void ShouldContinueMonitoring(object sender, ConditionalEventArgs e)
        {
            e.Result = !this.MonitoringCanceled;
        }

    }

}
