﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using WorkflowSimulator;
using System.Collections.ObjectModel;
using BlogMonitor;
using System.ServiceModel.Syndication;
using System.Workflow.Activities;

namespace BlogMonitorUI
{
    /// <summary>
    /// Interaction logic for BlogMonitorPanel.xaml
    /// </summary>
    public partial class BlogMonitorPanel : UserControl, IWorkflowUI
    {
        IEnumerable<string> _minutes = Enumerable.Range(0, 60).Select(i => i.ToString());

        public IEnumerable<string> Minutes
        {
            get { return _minutes; }
            set { _minutes = value; }
        }
        IEnumerable<string> _hours = Enumerable.Range(1, 12).Select(i => i.ToString());

        public IEnumerable<string> Hours
        {
            get { return _hours; }
            set { _hours = value; }
        }

        List<string> _amPm = new List<string> { "AM", "PM" };

        public List<string> AmPm
        {
            get { return _amPm; }
            set { _amPm = value; }
        }

        public ObservableCollection<FeedQuery> FeedQueries
        {
            get { return (ObservableCollection<FeedQuery>)GetValue(FeedQueriesProperty); }
            set { SetValue(FeedQueriesProperty, value); }
        }

        public static readonly DependencyProperty FeedQueriesProperty =
            DependencyProperty.Register("FeedQueries", typeof(ObservableCollection<FeedQuery>), typeof(BlogMonitorPanel), null);



        public WorkflowDiagram WorkflowDiagram
        {
            get;
            set;
        }

        public BlogMonitorPanel()
        {
            InitializeComponent();

            FeedQueries = new ObservableCollection<FeedQuery>() { new FeedQuery() { FeedUrl="http://www.microsoft.com/feeds/msdn/en-us/netframework/wf_en_us.xml", SearchKeyword="workflow" }};

            TimeSpan defaultTime = DateTime.Now.TimeOfDay;
            InitTimeCombos(defaultTime, lstCollectAtAmPm, lstCollectAtHour, lstCollectAtMinute);
            InitTimeCombos(defaultTime, lstSendAtAmPm, lstSendAtHour, lstSendAtMinute);
 
        }

        private void InitTimeCombos(TimeSpan defaultTime, ComboBox cboAmPm, ComboBox cboHour, ComboBox cboMinute)
        {
            cboAmPm.SelectedValue = defaultTime.Hours > 12 ? "PM" : "AM";

            if (defaultTime.Hours == 0)
            {
                cboHour.SelectedValue = "12";
            }
            else
            {
                cboHour.SelectedValue = defaultTime.Hours > 12 ? (defaultTime.Hours - 12).ToString() : defaultTime.Hours.ToString();
            }

            cboMinute.SelectedValue = defaultTime.Minutes.ToString();
            
        }


        private void btnAddFeed_Click(object sender, RoutedEventArgs e)
        {
            if (!string.IsNullOrEmpty(txtFeedUrl.Text) &&
                FeedQueries.All(fq => (fq.FeedUrl != txtFeedUrl.Text)))
            {
                FeedQueries.Add(new FeedQuery() { FeedUrl = txtFeedUrl.Text, SearchKeyword = txtFeedKeyword.Text });
                txtFeedUrl.Text = string.Empty;
                txtFeedKeyword.Text = string.Empty;
            }
        }

        private int ConvertHour(string hourValue, string amPmValue)
        {
            int resultHour;

            if (amPmValue == "AM")
            {
                resultHour = hourValue == "12" ? 0 : int.Parse(hourValue);
            }
            else
            {
                resultHour = hourValue == "12" ? 12 : int.Parse(hourValue) + 12;
            }

            return resultHour;
        }

        private void btnSubscribe_Click(object sender, RoutedEventArgs e)
        {
            TimeSpan collectAt = new TimeSpan(  ConvertHour((string)lstCollectAtHour.SelectedValue, (string)lstCollectAtAmPm.SelectedValue), 
                                                int.Parse((string)lstCollectAtMinute.SelectedValue), 
                                                0);

            TimeSpan sendAt = new TimeSpan( ConvertHour((string)lstSendAtHour.SelectedValue, (string)lstCollectAtAmPm.SelectedValue), 
                                            int.Parse((string)lstSendAtMinute.SelectedValue), 
                                            0);

            Dictionary<string, object> initialParameters = new Dictionary<string, object>()
            {
                {"FeedsToMonitor", FeedQueries.ToList()},
                {"CollectAt", collectAt},
                {"SendAt", sendAt}
            };
                 
            WorkflowDiagram.RunWorkflow(initialParameters);

            BlogMonitor.BlogMonitor blogMonitorService = WorkflowDiagram.LocalServiceInstance as BlogMonitor.BlogMonitor;
            blogMonitorService.ResultsAvailable += (obj, args) =>
            {
                ResultsPanel.Dispatcher.Invoke(new System.Windows.Forms.MethodInvoker(delegate()
                {
                    ResultsScrollViewer.Visibility = Visibility.Visible;
                    ResultsPanel.Children.Clear();

                    foreach (string feedUrl in args.Results.Keys)
                    {
                        FeedResult feedResult = args.Results[feedUrl];
                        TextBlock txtFeedTitle = new TextBlock() { Text = feedResult.FeedTitle, FontSize = 18, Margin = new Thickness(0, 15, 0, 0), TextWrapping = TextWrapping.Wrap };
                        TextBlock txtUrl = new TextBlock() { Text = feedUrl, FontSize = 8, TextWrapping = TextWrapping.Wrap };

                        ResultsPanel.Children.Add(txtFeedTitle);
                        ResultsPanel.Children.Add(txtUrl);

                        foreach (SyndicationItem item in feedResult.FeedItems)
                        {
                            TextBlock txtItemTitle = new TextBlock() { Text = item.Title.Text, FontSize = 14, Margin = new Thickness(0, 10, 0, 0), TextWrapping = TextWrapping.Wrap };
                            TextBlock txtItemSummary = new TextBlock() { Text = item.Summary.Text, TextWrapping = TextWrapping.Wrap };

                            ResultsPanel.Children.Add(txtItemTitle);
                            ResultsPanel.Children.Add(txtItemSummary);
                        }
                    }
                }));


            };

            AddFeedPanel.Visibility = Visibility.Collapsed;

            lstConfiguredFeeds.Visibility = Visibility.Collapsed;

            CollectAtPanel.Visibility = Visibility.Collapsed;

            SendAtPanel.Visibility = Visibility.Collapsed;

            btnSubscribe.Visibility = Visibility.Collapsed;
            btnUnsubscribe.Visibility = Visibility.Visible;
            txtPanelTitle.Text = "Monitoring Results";

        }

        private void btnUnsubscribe_Click(object sender, RoutedEventArgs e)
        {
            BlogMonitor.BlogMonitor blogMonitorService = WorkflowDiagram.LocalServiceInstance as BlogMonitor.BlogMonitor;
            blogMonitorService.OnUnsubscribe(new ExternalDataEventArgs(WorkflowDiagram.WorkflowInstance.InstanceId));

            txtPanelTitle.Text = "Monitoring Stopped";
            btnUnsubscribe.Visibility = Visibility.Collapsed;
            ResultsScrollViewer.Visibility = Visibility.Collapsed;
        }
    }
}
