﻿using System;
using System.Workflow.ComponentModel.Design;
using System.Drawing;
using System.Workflow.ComponentModel;
using System.Drawing.Drawing2D;
using System.Workflow.Activities;

namespace BugWorkflow
{
	class BugActivityDesigner: ActivityDesigner
	{
        Rectangle _textRectangle;
        Rectangle _imageRectangle;

        protected override void Initialize(System.Workflow.ComponentModel.Activity activity)
        {
            base.Initialize(activity);
        }

        protected override System.Drawing.Size OnLayoutSize(ActivityDesignerLayoutEventArgs e)
        {
            Font activeFont = e.DesignerTheme.Font;
            
            SizeF textAreaSize = e.Graphics.MeasureString(this.Text, activeFont, new SizeF(200, 200), this.StringFormat);

            if (textAreaSize.Width < 200)
            {
                textAreaSize.Width = 200;
            }

            LayoutContentRects(textAreaSize);

            Size actualSize = new Size((int)textAreaSize.Width, (int)textAreaSize.Height);
            actualSize.Height += 26;

            return actualSize;
        }

        private void LayoutContentRects(SizeF textAreaSize)
        {
            UpdateContentRectsLocation();
            _textRectangle.Size = textAreaSize.ToSize();
            _textRectangle.Inflate(-2, -2);

            Size imageRectSize = new Size((int)textAreaSize.Width, 26);
            _imageRectangle.Size = imageRectSize;
            _imageRectangle.Inflate(-2, -2);
        }

        private void UpdateContentRectsLocation()
        {
            //Most images are 16x16, add 5 units above and below to get 26.
            Point imageLocation = new Point(this.Location.X, this.Location.Y);
            imageLocation.Offset(this.Image.Width / 2 + 2, 2);
            _imageRectangle.Location = imageLocation; //new Rectangle(imageLocation, new Size());

            Point textLocation = new Point(this.Location.X, this.Location.Y);
            textLocation.Offset(2, 24);
            _textRectangle.Location = textLocation; //new Rectangle(textLocation, new Size());
            

        }

        protected StringFormat StringFormat
        {
            get
            {
                StringFormat stringFormat = new StringFormat();
                stringFormat.Alignment = StringAlignment.Center;
                stringFormat.Trimming = StringTrimming.EllipsisCharacter;
                stringFormat.LineAlignment = StringAlignment.Center;
                return stringFormat;
            }
        }

        protected override void OnActivityChanged(ActivityChangedEventArgs e)
        {
            base.OnActivityChanged(e);

            if (e.Member.Name == "Description" || e.Member.Name == "Name")
            {
                this.PerformLayout();
            }
        }

        protected override Rectangle ImageRectangle
        {
            get
            {
                return _imageRectangle;
            }
        }

        public override string Text
        {
            get
            {
                if (String.IsNullOrEmpty(this.Activity.Description))
                {
                    return this.Activity.Name;
                }
                else
                {
                    return this.Activity.Description;
                }
            }
            protected set
            {
                base.Text = value;
            }
        }

        protected override Rectangle TextRectangle
        {
            get
            {
                return _textRectangle;
            }
        }

        protected override void OnPaint(ActivityDesignerPaintEventArgs e)
        {
            Rectangle borderRect = this.Bounds;
            borderRect.Inflate(-2, -2);

            UpdateContentRectsLocation();

            GraphicsPath activityPath = ActivityDesignerPaint.GetRoundedRectanglePath(borderRect, 4);

            if (this.Activity is HandleExternalEventActivity)
            {
                e.Graphics.FillPath(System.Drawing.Brushes.Gold, activityPath);
            }
            else
            {
                e.Graphics.FillPath(System.Drawing.Brushes.White, activityPath);
            }

            ActivityDesignerPaint.DrawRoundedRectangle(e.Graphics, e.DesignerTheme.BorderPen, borderRect, 4); //Entire Activity
            ActivityDesignerPaint.DrawImage(e.Graphics, this.Image, this.ImageRectangle, DesignerContentAlignment.CenterLeft); 

            Activity activity = this.Activity as Activity;

            if (activity != null)
            {
                string headerText = activity.Name;
                StringFormat headerFormat = (StringFormat)this.StringFormat.Clone();
                headerFormat.Alignment = StringAlignment.Far;
                headerFormat.LineAlignment = StringAlignment.Center;
                headerFormat.FormatFlags = StringFormatFlags.NoWrap; 
                Rectangle headerTextRectangle = ImageRectangle;
                headerTextRectangle.Inflate(-this.Image.Width, -2);
                headerTextRectangle.Offset(this.Image.Width / 2 - 4, 0);

                e.Graphics.DrawString(headerText, DesignerTheme.BoldFont, Brushes.Black,
                    headerTextRectangle, headerFormat);

                e.Graphics.DrawString(this.Text, DesignerTheme.Font, Brushes.Black,
                    this.TextRectangle, this.StringFormat);
            }
        }


	}
}
