﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Forms;

namespace BugWorkflow
{
    /// <summary>
    /// Interaction logic for BugReviewPanel.xaml
    /// </summary>
    public partial class BugReviewPanel : System.Windows.Controls.UserControl, WorkflowSimulator.IWorkflowUI
    {
        public BugReport BugReport
        {
            get { return (BugReport)GetValue(BugReportProperty); }
            set { SetValue(BugReportProperty, value); }
        }

        public static readonly DependencyProperty BugReportProperty =
            DependencyProperty.Register("BugReport", typeof(BugReport), typeof(BugReviewPanel));

        public WorkflowSimulator.WorkflowDiagram WorkflowDiagram
        {
            get;
            set;
        }

        public BugReviewPanel()
        {
            InitializeComponent();

            this.BugReport = new BugReport()
            {
                AssignedToRole = "QA",
                AssignedToUser = "Ester Rester"
            };

            UpdateUIState();
        }

        private void UpdateUIState()
        {
            BugReport bugReport = this.BugReport;

            gridReport.DataContext = null;
            gridReport.DataContext = this.BugReport;

            btnSubmit.Visibility =  Visibility.Collapsed;
            btnClose.Visibility =   Visibility.Collapsed;
            btnApprove.Visibility = Visibility.Collapsed;
            btnReturn.Visibility =  Visibility.Collapsed;

            if (bugReport.Status == "Closed")
            {
                txtRole.Text = "CLOSED";
                return;
            }

            if (bugReport.AssignedToRole == "Development Manager")
            {
                btnApprove.Visibility = Visibility.Visible;
                btnReturn.Visibility = Visibility.Visible;
                btnClose.Visibility = Visibility.Visible;
            }
            else if (bugReport.AssignedToRole == "QA")
            {
                if (String.IsNullOrEmpty(bugReport.Status))
                {
                    btnSubmit.Visibility = Visibility.Visible;
                }
                else
                {
                    btnSubmit.Visibility = Visibility.Visible;
                    btnClose.Visibility = Visibility.Visible;
                }
            }
            else if (bugReport.AssignedToRole == "Developer")
            {
                btnReturn.Visibility = Visibility.Visible;
            }
        }

        private void btnClose_Click(object sender, RoutedEventArgs e)
        {
            gridReport.IsEnabled = false;
            this.BugReport.Status = "Closed";
            BugWorkflowService localService = this.WorkflowDiagram.LocalServiceInstance as BugWorkflowService;
            localService.OnClose(new BugReportEventArgs(this.BugReport, this.WorkflowDiagram.WorkflowInstance.InstanceId));
        }

        private void btnReturn_Click(object sender, RoutedEventArgs e)
        {
            gridReport.IsEnabled = false;
            BugWorkflowService localService = this.WorkflowDiagram.LocalServiceInstance as BugWorkflowService;
            localService.OnReturn(new BugReportEventArgs(this.BugReport, this.WorkflowDiagram.WorkflowInstance.InstanceId));
        }

        private void btnApprove_Click(object sender, RoutedEventArgs e)
        {
            gridReport.IsEnabled = false;
            BugWorkflowService localService = this.WorkflowDiagram.LocalServiceInstance as BugWorkflowService;
            localService.OnApprove(new BugReportEventArgs(this.BugReport, this.WorkflowDiagram.WorkflowInstance.InstanceId));
        }

        private void btnSubmit_Click(object sender, RoutedEventArgs e)
        {
            gridReport.IsEnabled = false;

            if (String.IsNullOrEmpty(this.BugReport.Status))
            {
                this.BugReport.Status = "Active";

                Dictionary<string, object> initParams = new Dictionary<string, object>()
                {
                    {"BugReport", this.BugReport}
                };

                this.WorkflowDiagram.RunWorkflow(initParams);

                BugWorkflowService localService = this.WorkflowDiagram.LocalServiceInstance as BugWorkflowService;
                localService.BugReportAssigned += new EventHandler<BugReportEventArgs>(localService_BugReportAssigned);

                this.WorkflowDiagram.WorkflowRuntime.WorkflowIdled += (obj, args) =>
                    {
                        gridReport.Dispatcher.Invoke((MethodInvoker)delegate
                        {
                            UpdateUIState();
                            gridReport.IsEnabled = true;
                        });
                    };

                this.WorkflowDiagram.WorkflowRuntime.WorkflowCompleted += (obj, args) =>
                    {
                        gridReport.Dispatcher.Invoke((MethodInvoker)delegate
                        {
                            this.BugReport.Status = "Closed";
                            UpdateUIState();
                        });
                    };
            }
            else
            {
                BugWorkflowService localService = this.WorkflowDiagram.LocalServiceInstance as BugWorkflowService;
                localService.OnSubmit(new BugReportEventArgs(this.BugReport, this.WorkflowDiagram.WorkflowInstance.InstanceId));
            }
        }

        void localService_BugReportAssigned(object sender, BugReportEventArgs e)
        {
            gridReport.Dispatcher.Invoke((MethodInvoker)delegate
            {
                this.BugReport = e.BugReport;
            });
        }
    }
}
