﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Configuration;
using System.Workflow.Runtime.Hosting;

namespace ConferenceRegistration
{
    public partial class ConferenceRegistrationPanel : UserControl, WorkflowSimulator.IWorkflowUI
    {
        public WorkflowSimulator.WorkflowDiagram WorkflowDiagram
        {
            get;
            set;
        }

        public string SqlPersistenceConnString
        {
            get
            {
                return ConfigurationManager.ConnectionStrings["SqlPersistenceStore"].ConnectionString;
            }
        }

        public ConferenceRegistrationPanel()
        {
            InitializeComponent();
        }


        private void btnRegister_Click(object sender, RoutedEventArgs e)
        {
            this.WorkflowDiagram.AfterCreateWorkflowRuntime += (o, args) =>
            {
            SqlWorkflowPersistenceService sqlPs = new SqlWorkflowPersistenceService(this.SqlPersistenceConnString, 
                true, 
                TimeSpan.FromMinutes(10), 
                TimeSpan.FromSeconds(1) 
                );

            this.WorkflowDiagram.WorkflowRuntime.AddService(sqlPs);
            };

            Address billingAddress = new Address()
            {
                Street1 = txtStreet.Text,
                City = txtCity.Text,
                State = txtState.Text,
                Zip = txtZip.Text,
                Phone = txtPhone.Text
            };

            Billing billingInfo = new Billing()
            {
                NameOnCard = txtNameOnCard.Text,
                BillingAddress = billingAddress,
                CreditCardNumber = txtCcNum.Text,
                CreditCardSecurityCode = txtCiv.Text,
                ExpirationDate = txtExpiration.Text
            };

            Customer customerInfo = new Customer()
            {
                FirstName = txtFirstName.Text,
                LastName = txtLastName.Text,
                Email = txtEmail.Text

            };

            Dictionary<string, object> initParameters = new Dictionary<string, object>()
            {
                {   
                    "AttendeeRegistration", 
                    new RegistrationInfo()
                    {
                        BillingInfo = billingInfo,
                        CustomerInfo = customerInfo 
                    }
                }
            };

            WorkflowDiagram.RunWorkflow(initParameters);

            WorkflowDiagram.WorkflowRuntime.WorkflowPersisted += (o, w) =>
                {
                    MessageBox.Show("Workflow Persisted.");
                };

            WorkflowDiagram.WorkflowRuntime.WorkflowUnloaded += (o, w) =>
            {
                MessageBox.Show("Workflow Unloaded.");
            };

            WorkflowDiagram.WorkflowRuntime.WorkflowLoaded += (o, w) =>
                {
                    MessageBox.Show("Workflow Loaded.");
                };
        }
    }
}
