﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using WorkflowSimulator;
using System.Workflow.Runtime.Tracking;
using System.Configuration;
using System.Workflow.ComponentModel;
using System.Workflow.Activities;
using System.Workflow.ComponentModel.Compiler;

namespace Crm
{
    /// <summary>
    /// Interaction logic for CrmPanel.xaml
    /// </summary>
    public partial class CrmPanel : UserControl, IWorkflowUI
    {
        public WorkflowDiagram WorkflowDiagram
        {
            get;
            set;
        }

        public string SqlTrackingConnString
        {
            get
            {
                return ConfigurationManager.ConnectionStrings["SqlTrackingStore"].ConnectionString;
            }
        }

        public SalesLead SalesLead
        {
            get { return (SalesLead)GetValue(SalesLeadProperty); }
            set { SetValue(SalesLeadProperty, value); }
        }

        public static readonly System.Windows.DependencyProperty SalesLeadProperty =
            System.Windows.DependencyProperty.Register("SalesLead", typeof(SalesLead), typeof(CrmPanel));

        public CrmPanel()
        {
            InitializeComponent();

            this.SalesLead = new SalesLead()
            {
                SalesPerson = "Henry Bank",
                CustomerEmail = "fad@hotmail.com",
                CustomerName = "Frank Customer",
                CustomerPhone = "878-579-4563",
                Notes = "Seemed really interested in widgets."
            };

            rootLayout.DataContext = this.SalesLead;
        }

        private void btnSubmitLead_Click(object sender, RoutedEventArgs e)
        {
            btnSubmitLead.Visibility = Visibility.Collapsed;
            btnDeal.Visibility = Visibility.Visible;
            btnNoDeal.Visibility = Visibility.Visible;
            btnNoResponse.Visibility = Visibility.Visible;

            grdDynamicUpdate.Visibility = Visibility.Visible;
            chkAllowDynamicUpdate.IsEnabled = false;

            txtPanelTitle.Text = "Sales Followup";

            this.WorkflowDiagram.AfterCreateWorkflowRuntime+=(o, args) =>
                {
                    this.WorkflowDiagram.WorkflowRuntime.AddService(
                        new SqlTrackingService(this.SqlTrackingConnString));
                };

            Dictionary<string, object> initialParameters = new Dictionary<string, object>()
            {
                {"SalesLead", this.SalesLead},
                {"AllowUpdates", chkAllowDynamicUpdate.IsChecked}
            };

            this.WorkflowDiagram.RunWorkflow(initialParameters);
        }

        private void btnDeal_Click(object sender, RoutedEventArgs e)
        {
            CrmService crmService = this.WorkflowDiagram.LocalServiceInstance as CrmService;
            crmService.OnDeal(this.WorkflowDiagram.WorkflowInstance.InstanceId);
        }

        private void btnNoDeal_Click(object sender, RoutedEventArgs e)
        {
            CrmService crmService = this.WorkflowDiagram.LocalServiceInstance as CrmService;
            crmService.OnNoDeal(this.WorkflowDiagram.WorkflowInstance.InstanceId);
        }

        private void btnNoResponse_Click(object sender, RoutedEventArgs e)
        {
            CrmService crmService = this.WorkflowDiagram.LocalServiceInstance as CrmService;
            crmService.OnNoResponse(this.WorkflowDiagram.WorkflowInstance.InstanceId);
        }

        private void btnAddExpiration_Click(object sender, RoutedEventArgs e)
        {
            WorkflowChanges workflowChanges = new WorkflowChanges(this.WorkflowDiagram.WorkflowInstance.GetWorkflowDefinition());

            if ((workflowChanges.TransientWorkflow as SequentialWorkflowActivity) != null)
            {
                UpdateSequentialVersion(workflowChanges);
            }
            else
            {
                UpdateStateMachineVersion(workflowChanges);
            }

            ValidationErrorCollection errors = workflowChanges.Validate();

            if (!errors.HasErrors)
            {
                try
                {
                    this.WorkflowDiagram.WorkflowInstance.ApplyWorkflowChanges(workflowChanges);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Failed to Apply Workflow Changes", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
            else
            {
                var validationErrors = from err in errors
                                       where !err.IsWarning
                                       select err;

                StringBuilder sb = new StringBuilder();

                sb.AppendLine("The following errors were encountered:\n");
                foreach (System.Workflow.ComponentModel.Compiler.ValidationError err in validationErrors)
                {
                    sb.AppendLine(err.ErrorText);
                }

                MessageBox.Show(sb.ToString(), "Validation Errors Occured", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void UpdateSequentialVersion(WorkflowChanges workflowChanges)
        {
            ListenActivity listenActivity1 = workflowChanges.TransientWorkflow.GetActivityByName("listenActivity1") as ListenActivity;

            EventDrivenActivity expiryEventDriven = listenActivity1.GetActivityByName("expiryEventDriven") as EventDrivenActivity;

            if (expiryEventDriven != null)
            {
                //We are updating a workflow that already has the expiry logic
                listenActivity1.Activities.Remove(workflowChanges.TransientWorkflow.GetActivityByName("expiryEventDriven"));
            }

            expiryEventDriven = new EventDrivenActivity("expiryEventDriven");

            DelayActivity expiryDelay = new DelayActivity("WaitForExpiry");
            expiryDelay.TimeoutDuration = TimeSpan.Parse(txtDelay.Text);

            TrackData trackExpired = new TrackData();
            trackExpired.Name = "TrackExpired";
            trackExpired.UserDataKey = "Closed-LeadExpired";
            trackExpired.UserDataValue = "System";

            expiryEventDriven.Activities.Add(expiryDelay);

            expiryEventDriven.Activities.Add(trackExpired);

            listenActivity1.Activities.Add(expiryEventDriven);
        }

        private void UpdateStateMachineVersion(WorkflowChanges workflowChanges)
        {
            StateActivity initialState = workflowChanges.TransientWorkflow.GetActivityByName("InitialState") as StateActivity;

            EventDrivenActivity expiryEventDriven = initialState.GetActivityByName("expiryEventDriven") as EventDrivenActivity;

            if (expiryEventDriven != null)
            {
                //We are updating a workflow that already has the expiry logic
                initialState.Activities.Remove(workflowChanges.TransientWorkflow.GetActivityByName("expiryEventDriven"));
            }

            expiryEventDriven = new EventDrivenActivity("expiryEventDriven");

            DelayActivity expiryDelay = new DelayActivity("WaitForExpiry");
            expiryDelay.TimeoutDuration = TimeSpan.Parse(txtDelay.Text);

            TrackData trackExpired = new TrackData();
            trackExpired.Name = "TrackExpired";
            trackExpired.UserDataKey = "Closed-LeadExpired";
            trackExpired.UserDataValue = "System";

            SetStateActivity setExpiryComplete = new SetStateActivity("SetExpiryComplete");
            setExpiryComplete.TargetStateName = "Complete";

            expiryEventDriven.Activities.Add(expiryDelay);

            expiryEventDriven.Activities.Add(trackExpired);

            expiryEventDriven.Activities.Add(setExpiryComplete);

            initialState.Activities.Add(expiryEventDriven);
        }
    }
}
