﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using WorkflowSimulator;
using System.Configuration;
using System.Workflow.Runtime.Hosting;
using System.Windows.Forms;

namespace ImageArchival
{
    /// <summary>
    /// Interaction logic for UserControl1.xaml
    /// </summary>
    public partial class ImageArchivalPanel : System.Windows.Controls.UserControl, IWorkflowUI
    {
        public WorkflowDiagram WorkflowDiagram
        {
            get;
            set;
        }

        public string SqlPersistenceConnString
        {
            get
            {
                return ConfigurationManager.ConnectionStrings["SqlPersistenceStore"].ConnectionString;
            }
        }

        public ImageArchivalPanel()
        {
            InitializeComponent();
        }

        private void btnLaunch_Click(object sender, RoutedEventArgs e)
        {
            this.WorkflowDiagram.AfterCreateWorkflowRuntime += (o, args) =>
            {
                this.WorkflowDiagram.WorkflowRuntime.AddService(
                    new SqlWorkflowPersistenceService(this.SqlPersistenceConnString));
            };

            string imageFileName = System.IO.Path.GetFileName(txtFilePath.Text);
            string imageFilePath = txtFilePath.Text;
            string archivePath = System.IO.Path.Combine(txtArchiveFolderPath.Text, imageFileName);
            System.IO.FileInfo fileInfo = new System.IO.FileInfo(imageFilePath);
            
            this.WorkflowDiagram.RunWorkflow(
                new Dictionary<string, object>()
                {
                    {"FileName", imageFileName},
                    {"SourcePath", imageFilePath},
                    {"ArchivePath", archivePath},
                    {"FileDate", fileInfo.CreationTime},
                    {"FileSize", fileInfo.Length},
                    {"Caption", txtCaption.Text},
                    {"Description", txtDescription.Text},
                    {"Username", System.Threading.Thread.CurrentPrincipal.Identity.Name},
                    {"SimulateSqlError", rdoSqlError.IsChecked},
                    {"SimulateCompensatableError", rdoCompError.IsChecked }
                }

                );
        }

        private void btnBrowseSourceFile_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                txtFilePath.Text = ofd.FileName; 
            }
        }

        private void btnBrowseArchiveFolderPath_Click(object sender, RoutedEventArgs e)
        {
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            if (fbd.ShowDialog() == DialogResult.OK)
            {
                txtArchiveFolderPath.Text = fbd.SelectedPath; 
            }
        }

    }
}
