﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using JobReqLocalService;

namespace JobReqUI
{
    /// <summary>
    /// Interaction logic for JobReqReviewPanel.xaml
    /// </summary>
    public partial class JobReqReviewPanel : UserControl, WorkflowSimulator.IWorkflowUI
    {
        string _currentStatus;

        public WorkflowSimulator.WorkflowDiagram WorkflowDiagram
        {
            get;
            set;
        }

        public JobReqReviewPanel()
        {
            InitializeComponent();
        }

        private void btnApprove_Click(object sender, RoutedEventArgs e)
        {
            SubmitJobReqReviewService submitJobReqReviewService = WorkflowDiagram.LocalServiceInstance as SubmitJobReqReviewService;

            if (submitJobReqReviewService != null)
            {
                if (_currentStatus == "Awaiting Senior Manager Review")
                {
                    _currentStatus = "Awaiting HR Review";
                }
                else if (_currentStatus == "Awaiting HR Review")
                {
                    _currentStatus = "Approved";
                }

                btnApprove.Visibility = Visibility.Collapsed;
                btnDeny.Visibility = Visibility.Collapsed; 

                submitJobReqReviewService.OnSubmitJobReqReview(new ReviewData("Approved", WorkflowDiagram.WorkflowInstance.InstanceId));  
            }
        }

        private void btnDeny_Click(object sender, RoutedEventArgs e)
        {
            SubmitJobReqReviewService submitJobReqReviewService = WorkflowDiagram.LocalServiceInstance as SubmitJobReqReviewService;

            if (submitJobReqReviewService != null)
            {
                _currentStatus = "Denied";

                btnApprove.Visibility = Visibility.Collapsed;
                btnDeny.Visibility = Visibility.Collapsed;

                submitJobReqReviewService.OnSubmitJobReqReview(new ReviewData("Denied", WorkflowDiagram.WorkflowInstance.InstanceId)); 
            }
        }

        private void btnSubmit_Click(object sender, RoutedEventArgs e)
        {
            btnSubmit.Visibility = Visibility.Collapsed;
            txtJobCode.IsEnabled = false;
            txtJobDescription.IsEnabled = false;
            txtPayGrade.IsEnabled = false;
            txtRequestedBy.IsEnabled = false;
            _currentStatus = "Awaiting Senior Manager Review";

            Dictionary<string, object> initParameters = new Dictionary<string, object>(){
                {"JobCode", txtJobCode.Text},
                {"JobDescription", txtJobDescription.Text},
                {"PayGrade", txtPayGrade.Text},
                {"RequestedBy", txtRequestedBy.Text},
                {"CurrentStatus", _currentStatus},
            };

            WorkflowDiagram.RunWorkflow(initParameters);

            WorkflowDiagram.WorkflowRuntime.WorkflowIdled += new EventHandler<System.Workflow.Runtime.WorkflowEventArgs>(WorkflowRuntime_WorkflowIdled);
            WorkflowDiagram.WorkflowRuntime.WorkflowCompleted += new EventHandler<System.Workflow.Runtime.WorkflowCompletedEventArgs>(WorkflowRuntime_WorkflowCompleted);
        }

        void WorkflowRuntime_WorkflowCompleted(object sender, System.Workflow.Runtime.WorkflowCompletedEventArgs e)
        {
            btnSubmit.Dispatcher.Invoke((System.Windows.Forms.MethodInvoker)delegate
            {
                txtTitle.Text = "Job Requisition - " + _currentStatus;
                btnApprove.Visibility = Visibility.Collapsed;
                btnDeny.Visibility = Visibility.Collapsed;
            });
        }

        void WorkflowRuntime_WorkflowIdled(object sender, System.Workflow.Runtime.WorkflowEventArgs e)
        {
            btnSubmit.Dispatcher.Invoke((System.Windows.Forms.MethodInvoker)delegate
            {
                txtTitle.Text = "Job Requisition - " + _currentStatus; 
                btnApprove.Visibility = Visibility.Visible;
                btnDeny.Visibility = Visibility.Visible;  
            });

        }
    }
}
