﻿using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Drawing;
using System.Linq;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Design;
using System.Workflow.ComponentModel.Compiler;
using System.Workflow.ComponentModel.Serialization;
using System.Workflow.Runtime;
using System.Workflow.Activities;
using System.Workflow.Activities.Rules;
using System.Collections.Generic;

namespace MerchantService
{
    public partial class OnItemPurchased : System.Workflow.ComponentModel.Activity, IEventActivity, IActivityEventListener<QueueEventArgs>
    {
        public Item Item
        {
            get { return (Item)GetValue(ItemProperty); }
            set { SetValue(ItemProperty, value); }
        }

        public static readonly DependencyProperty ItemProperty =
            DependencyProperty.Register("Item", typeof(Item), typeof(OnItemPurchased));

        public List<Item> PurchasedItems
        {
            get { return (List<Item>)GetValue(PurchasedItemsProperty); }
            set { SetValue(PurchasedItemsProperty, value); }
        }

        public static readonly DependencyProperty PurchasedItemsProperty =
            DependencyProperty.Register("PurchasedItems", typeof(List<Item>), typeof(OnItemPurchased));

        public decimal Total
        {
            get { return (decimal)GetValue(TotalProperty); }
            set { SetValue(TotalProperty, value); }
        }

        public static readonly DependencyProperty TotalProperty =
            DependencyProperty.Register("Total", typeof(decimal), typeof(OnItemPurchased));

        public string OrderId
        {
            get { return (string)GetValue(OrderIdProperty); }
            set { SetValue(OrderIdProperty, value); }
        }

        public static readonly DependencyProperty OrderIdProperty =
            DependencyProperty.Register("OrderId", typeof(string), typeof(OnItemPurchased));

        public OnItemPurchased()
        {
            InitializeComponent();
        }

        protected override void Initialize(IServiceProvider provider)
        {
            this.QueueName = OrderId;
        }

        #region IEventActivity Members

        [Browsable(false)]
        public IComparable QueueName
        {
            get;
            internal set;
        }

        public void Subscribe(ActivityExecutionContext parentContext, IActivityEventListener<QueueEventArgs> parentEventHandler)
        {
            WorkflowQueue workflowQueue = CreateQueue(parentContext);

            workflowQueue.RegisterForQueueItemAvailable(parentEventHandler);

        }

        public void Unsubscribe(ActivityExecutionContext parentContext, IActivityEventListener<QueueEventArgs> parentEventHandler)
        {
            WorkflowQueuingService qService = parentContext.GetService<WorkflowQueuingService>();

            if (qService.Exists(this.QueueName))
            {
                WorkflowQueue workflowQueue = qService.GetWorkflowQueue(this.QueueName);
                workflowQueue.UnregisterForQueueItemAvailable(parentEventHandler);
            }
        }

        #endregion

        #region IActivityEventListener<QueueEventArgs> Members

        public void OnEvent(object sender, QueueEventArgs e)
        {
            // If activity is not scheduled for execution, do nothing
            if (this.ExecutionStatus == ActivityExecutionStatus.Executing)
            {
                ActivityExecutionContext context = sender as ActivityExecutionContext;
                if (this.ProcessQueueItem(context))
                {
                    context.CloseActivity();
                }
            }
        }

        #endregion

        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            if (ProcessQueueItem(executionContext))
            {
                return ActivityExecutionStatus.Closed;
            }

            Subscribe(executionContext, this);
            return ActivityExecutionStatus.Executing;
        }

        protected override ActivityExecutionStatus Cancel(ActivityExecutionContext context)
        {
            Unsubscribe(context, this);
            DeleteQueue(context);
            return ActivityExecutionStatus.Closed;
        }

        private WorkflowQueue CreateQueue(ActivityExecutionContext context)
        {
            WorkflowQueuingService qService = context.GetService<WorkflowQueuingService>();

            if (!qService.Exists(this.QueueName))
            {
                qService.CreateWorkflowQueue(this.QueueName, true);
            }

            return qService.GetWorkflowQueue(this.QueueName);
        }

        private bool ProcessQueueItem(ActivityExecutionContext context)
        {
            WorkflowQueuingService qService = context.GetService<WorkflowQueuingService>();
            if (!qService.Exists(this.QueueName))
            {
                return false;
            }

            WorkflowQueue queue = qService.GetWorkflowQueue(this.QueueName);
            if (queue.Count == 0)
            {
                return false;
            }

            Item purchasedItem = queue.Dequeue() as Item;
            this.Item = purchasedItem;
            this.PurchasedItems.Add(Item);
            this.Total += purchasedItem.ItemCost; 

            Unsubscribe(context, this);
            DeleteQueue(context);
            return true;
        }

        private void DeleteQueue(ActivityExecutionContext context)
        {
            WorkflowQueuingService qService = context.GetService<WorkflowQueuingService>();
            qService.DeleteWorkflowQueue(this.QueueName);
        }
    }
}
