﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace SchedulingAssistant
{
	public static class MasterSchedule
	{
        public static bool IsFree(string name, AppointmentRequest appointmentRequest, string source)
        {
            bool returnValue = false; 
                
            switch (name)
            {
                case "Chris Smith":
                    returnValue = true;
                    break;
                case "George Brown":
                    returnValue = true;
                    break;
                case "Never There":
                    returnValue = false;
                    break;
                case "SuggestSchedules CommunicationException":
                    if (source == "SuggestSchedules")
                    {
                        throw new CalendarServerCommunicationException("Server actively refused the connection.");
                    }
                    else
                    {
                        returnValue = false;
                    }
                    break;
                case "ProcessFreeBusy CommunicationException":
                    if (source == "ProcessFreeBusy")
                    {
                        throw new CalendarServerCommunicationException("Server actively refused the connection.");
                    }
                    else
                    {
                        returnValue = false;
                    }
                    break;
                case "NotAllowed Exception":
                    if (source == "ProcessFreeBusy")
                    {
                        throw new CalendarAccessNotAllowedException(name);
                    }
                    else
                    {
                        returnValue = false;
                    }
                    break;
            }

            return returnValue;
                
        }

        public static FreeBusyResults SuggestAlternatives(AppointmentRequest appointmentRequest)
        {
            FreeBusyResults fbr;

            if ((appointmentRequest.Attendees.Count(a => a.Name == "NotAllowed Exception" && a.Required ) > 0) &&
                (appointmentRequest.Attendees.Count(a => a.Name == "Never There" && a.Required) > 0))
            {
                fbr = new FreeBusyResults()
                {
                    AlternativeAppointments = new ObservableCollection<AppointmentAlternative>()
                    {
                        new AppointmentAlternative()
                        {
                            Start = appointmentRequest.Start,
                            End = appointmentRequest.End,
                            UpdatedAttendees = new ObservableCollection<Attendee>()
                            {
                                new Attendee() { Name = "NotAllowed Exception", Required = false },
                                new Attendee() { Name = "Never There", Required = false }
                            }
                        },
                        new AppointmentAlternative()
                        {
                            Start = appointmentRequest.Start.AddHours(1),
                            End = appointmentRequest.End.AddHours(1),
                            UpdatedAttendees = new ObservableCollection<Attendee>()
                            {
                                new Attendee() { Name = "NotAllowed Exception", Required = false }
                            }
                        },
                        new AppointmentAlternative()
                        {
                            Start = appointmentRequest.Start.AddHours(2),
                            End = appointmentRequest.End.AddHours(2),
                            UpdatedAttendees = new ObservableCollection<Attendee>()
                            {
                                new Attendee() { Name = "NotAllowed Exception", Required = false }
                            }
                        }
                    }
                };
            }
            else if (appointmentRequest.Attendees.Count(a => a.Name == "NotAllowed Exception" && a.Required) > 0)
            {
                fbr = new FreeBusyResults()
                {
                    AlternativeAppointments = new ObservableCollection<AppointmentAlternative>()
                    {
                        new AppointmentAlternative()
                        {
                            Start = appointmentRequest.Start,
                            End = appointmentRequest.End,
                            UpdatedAttendees = new ObservableCollection<Attendee>()
                            {
                                new Attendee() { Name = "NotAllowed Exception", Required = false }
                            }
                        },
                        new AppointmentAlternative()
                        {
                            Start = appointmentRequest.Start.AddHours(1),
                            End = appointmentRequest.End.AddHours(1),
                            UpdatedAttendees = new ObservableCollection<Attendee>()
                            {
                                new Attendee() { Name = "NotAllowed Exception", Required = false }
                            }
                        },
                        new AppointmentAlternative()
                        {
                            Start = appointmentRequest.Start.AddHours(2),
                            End = appointmentRequest.End.AddHours(2),
                            UpdatedAttendees = new ObservableCollection<Attendee>()
                            {
                                new Attendee() { Name = "NotAllowed Exception", Required = false }
                            }
                        }
                    }
                };
            }
            else if (appointmentRequest.Attendees.Count(a => a.Name == "Never There" && a.Required) > 0)
            {
                fbr = new FreeBusyResults()
                {
                    AlternativeAppointments = new ObservableCollection<AppointmentAlternative>()
                    {
                        new AppointmentAlternative()
                        {
                            Start = appointmentRequest.Start.AddHours(1),
                            End = appointmentRequest.End.AddHours(1)
                        },
                        new AppointmentAlternative()
                        {
                            Start = appointmentRequest.Start.AddHours(2),
                            End = appointmentRequest.End.AddHours(2)
                        },
                        new AppointmentAlternative()
                        {
                            Start = appointmentRequest.Start.AddHours(3),
                            End = appointmentRequest.End.AddHours(3)
                        },
                        new AppointmentAlternative()
                        {
                            Start = appointmentRequest.Start,
                            End = appointmentRequest.End,
                            UpdatedAttendees = new ObservableCollection<Attendee>()
                            {
                                new Attendee() { Name = "Never There", Required = false }
                            }
                        },
                        new AppointmentAlternative()
                        {
                            Start = appointmentRequest.Start.AddHours(1),
                            End = appointmentRequest.End.AddHours(1),
                            UpdatedAttendees = new ObservableCollection<Attendee>()
                            {
                                new Attendee() { Name = "Never There", Required = false }
                            }
                        },
                        new AppointmentAlternative()
                        {
                            Start = appointmentRequest.Start.AddHours(2),
                            End = appointmentRequest.End.AddHours(2),
                            UpdatedAttendees = new ObservableCollection<Attendee>()
                            {
                                new Attendee() { Name = "Never There", Required = false }
                            }
                        }
                    }
                };
            }
            else
            {
                fbr = new FreeBusyResults();
            }

            return fbr;
        }
	}
}
