﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace SchedulingAssistant
{
    [Serializable]
    public class AppointmentRequest
    {
        public string Subject { get; set; }
        public string Location {get;set;}
        public DateTime Start { get; set; }
        public DateTime End { get; set; }
        public ObservableCollection<Attendee> Attendees { get; set; }
    }

    [Serializable]
    public class Attendee
    {
        public string Name { get; set; }
        public string Email { get; set; }
        public bool Required { get; set; }
    }

    [Serializable]
    public class AppointmentAlternative
    {
        public DateTime Start { get; set; }
        public DateTime End { get; set; }
        public ObservableCollection<Attendee> UpdatedAttendees { get; set; }
        public string Reason 
        {
            get
            {
                if (UpdatedAttendees == null)
                    return "";

                StringBuilder sb = new StringBuilder();

                foreach (Attendee attendee in UpdatedAttendees)
                {
                    sb.Append(attendee.Name + ", "); 
                }
                sb.Remove(sb.Length - 2, 2);

                return sb.ToString();
            }
        }
    }

    [Serializable]
    public class FreeBusyResults
    {
        public Exception Fault { get; set; }
        public ObservableCollection<AppointmentAlternative> AlternativeAppointments { get; set; }
    }

    public class CalendarAccessNotAllowedException : Exception
    {
        public string User 
        { 
            get; 
            protected set; 
        }

        public override string Message
        {
            get
            {
                return string.Format("Calendar access not allowed for user '{0}'.", User); ;
            }
        }

        public CalendarAccessNotAllowedException(string user):base()
        {
            this.User = user;
        }
    }

    public class CalendarServerCommunicationException : Exception
    {
        public CalendarServerCommunicationException(string reason)
            : base(reason)
        { }

    }
}
