﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Windows.Forms;

namespace SchedulingAssistant
{
    /// <summary>
    /// Interaction logic for SchedulingAssistantPanel.xaml
    /// </summary>
    public partial class SchedulingAssistantPanel : System.Windows.Controls.UserControl, WorkflowSimulator.IWorkflowUI
    {
        public WorkflowSimulator.WorkflowDiagram WorkflowDiagram
        {
            get;
            set;
        }

        public SchedulingAssistantPanel()
        {
            InitializeComponent();

            this.AvailableAttendees = new ObservableCollection<Attendee>()
            {
                new Attendee() { Name = "George Brown", Required = true } ,
                new Attendee() { Name = "Never There", Required = true } ,
                new Attendee() { Name = "NotAllowed Exception", Required = true } ,
                new Attendee() { Name = "ProcessFreeBusy CommunicationException", Required = true } ,
                new Attendee() { Name = "SuggestSchedules CommunicationException", Required = true }
            };

            this.AppointmentRequest = new AppointmentRequest()
            {
                Subject = "Code review communication framework",
                Location = "Conference room",
                Start = DateTime.Now.AddHours(1),
                End = DateTime.Now.AddHours(2),
                Attendees = new ObservableCollection<Attendee>()
                {
                    new Attendee() { Name = "Chris Smith", Required = true } 
                }
            };

            this.DataContext = this.AppointmentRequest;
            lstAvailableAttendees.ItemsSource = this.AvailableAttendees;
            lstSelectedAttendees.ItemsSource = this.AppointmentRequest.Attendees;
        }

        public AppointmentRequest AppointmentRequest 
        {
            get;
            set;
        }

        public FreeBusyResults FreeBusyResults
        {
            get;
            set;
        }

        public ObservableCollection<Attendee> AvailableAttendees
        {
            get;
            set;
        }

        private void btnSubmit_Click(object sender, RoutedEventArgs e)
        {

            Dictionary<string, object> initParameters = new Dictionary<string,object>()
            {
                {"AppointmentRequest", AppointmentRequest}
            };

            WorkflowDiagram.RunWorkflow(initParameters);

            WorkflowDiagram.WorkflowRuntime.WorkflowCompleted += (obj, args) =>
                {
                    FreeBusyResults = args.OutputParameters["FreeBusyResults"] as FreeBusyResults;

                    lstSuggestions.Dispatcher.Invoke((MethodInvoker)delegate()
                    {
                        if (FreeBusyResults == null)
                        {
                            lstSuggestions.Visibility = Visibility.Collapsed;
                            txtMessage.Text = "All attendees were available. Appointment requests sent.";
                        }
                        else if (FreeBusyResults.Fault != null)
                        {
                            lstSuggestions.Visibility = Visibility.Collapsed;
                            txtMessage.Text = FreeBusyResults.Fault.Message;
                        }
                        else
                        {
                            lstSuggestions.Visibility = Visibility.Visible;
                            lstSuggestions.ItemsSource = FreeBusyResults.AlternativeAppointments;
                            txtMessage.Text = "Here are some suggested alternative schedules.";
                        }
                    });
                };
        }

        private void btnAddAttendee_Click(object sender, RoutedEventArgs e)
        {
            Attendee selectedItem = lstAvailableAttendees.SelectedItem as Attendee;
            if (selectedItem == null)
                return;

            AppointmentRequest.Attendees.Add(selectedItem);
            AvailableAttendees.Remove(selectedItem);
        }

        private void btnRemoveAttendee_Click(object sender, RoutedEventArgs e)
        {
            Attendee selectedItem = lstSelectedAttendees.SelectedItem as Attendee;
            if (selectedItem == null)
                return;

             AvailableAttendees.Add(selectedItem);
             AppointmentRequest.Attendees.Remove(selectedItem);
        }
    }
}
