﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Workflow.Runtime.Tracking;

namespace WorkflowSimulator
{
    public class SimulationTrackingChannel: TrackingChannel
    {
        private TrackingParameters _trackingParameters = null;

        public event EventHandler<ActivityTrackingEventArgs> ActivityStatusUpdate;

        public SimulationTrackingChannel(TrackingParameters parameters)
        {
            _trackingParameters = parameters;
        }

        protected override void InstanceCompletedOrTerminated()
        {
            //Not currently used.
        }

        protected override void Send(TrackingRecord record)
        {
            if (record is ActivityTrackingRecord)
            {
                OnActivityStatusUpdate((ActivityTrackingRecord)record);
            }
            else if (record is WorkflowTrackingRecord)
            {
                //Not currently used.
            }
            else if (record is UserTrackingRecord)
            {
                //Not currently used.
            }
        }

        protected void OnActivityStatusUpdate(ActivityTrackingRecord activityTrackingRecord)
        {
            if (ActivityStatusUpdate != null)
            {
                ActivityStatusUpdate(this, new ActivityTrackingEventArgs(activityTrackingRecord));
            }
        }
    }

    public class ActivityTrackingEventArgs : EventArgs
    {
        public ActivityTrackingRecord ActivityTrackingRecord { get; set; }

        public ActivityTrackingEventArgs(ActivityTrackingRecord activityTrackingRecord): base()
        {
            this.ActivityTrackingRecord = activityTrackingRecord;
        }
    }


}
