using System;
using System.Collections.Generic;
using System.Text;
using System.Workflow.ComponentModel.Design;
using System.ComponentModel.Design.Serialization;
using System.ComponentModel.Design;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Compiler;
using System.Drawing.Design;


namespace WorkflowSimulator
{
    public class WorkflowDefinitionLoader: WorkflowDesignerLoader
    {
        private Activity _workflowDefinition;

        public Activity WorkflowDefinition
        {
            get { return _workflowDefinition; }
            set { _workflowDefinition = value; }
        }

        private ITypeProvider _typeProvider;

        public ITypeProvider TypeProvider
        {
            get { return _typeProvider; }
            set { _typeProvider = value; }
        }
	
        #region WorkflowDesignerLoader abstract member implementation 

        public override string FileName
        {
            get { return null; }
        }

        public override System.IO.TextReader GetFileReader(string filePath)
        {
            return null;
        }

        public override System.IO.TextWriter GetFileWriter(string filePath)
        {
            return null;
        }

        #endregion

        protected override void Initialize()
        {
            base.Initialize();

            IDesignerLoaderHost designerLoaderHost = this.LoaderHost;

            if (designerLoaderHost != null)
            {
                designerLoaderHost.AddService(typeof(IMenuCommandService), new WorkflowMenuService(designerLoaderHost));

                designerLoaderHost.AddService(typeof(ITypeProvider), _typeProvider, true);

                
            }

        }

        protected override void PerformLoad(IDesignerSerializationManager serializationManager)
        {
            base.PerformLoad(serializationManager);

            IDesignerHost designerHost = GetService(typeof(IDesignerHost)) as IDesignerHost;
            AddWorkflowToDesigner(designerHost, this._workflowDefinition);
            designerHost.Activate();
        }

        protected override void OnEndLoad(bool successful, System.Collections.ICollection errors)
        {
            base.OnEndLoad(successful, errors);

            System.Collections.IList errList;

            if (successful)
            {
                //Load state machine layouts if available
                Type wfType = this.WorkflowDefinition.GetType();
                string layoutResourceName = String.Format("{0}.layout", wfType.Name);
                this.LoadDesignerLayoutFromResource(wfType, layoutResourceName, out errList);   

            }
            else
            {
                if (errors != null)
                {
                    StringBuilder sb = new StringBuilder();
                    foreach (string error in errors)
                    {
                        sb.AppendLine(error);
                    }
                    throw new ApplicationException(String.Format("Unable to load workflow, the following error(s) occured:\n{0}", sb.ToString()));
                }
            }
        }

        private static void AddWorkflowToDesigner(IDesignerHost designer, Activity workflow)
        {
            designer.Container.Add(workflow, workflow.QualifiedName);

            if (workflow is CompositeActivity)
            {
                List<Activity> children = new List<Activity>();

                GetChildActivities(workflow as CompositeActivity, children);

                foreach (Activity child in children)
                {
                    designer.Container.Add(child, child.QualifiedName);
                }
            }
        }

        private static void GetChildActivities(CompositeActivity composite, List<Activity> children)
        {
            foreach (Activity activity in composite.Activities)
            {
                children.Add(activity);

                if (activity is CompositeActivity)
                {
                    GetChildActivities(activity as CompositeActivity, children);
                }
            }
        }

        public void RemoveFromDesigner(IDesignerHost designer, Activity workflow)
        {
            if (workflow != null)
            {
                designer.DestroyComponent(workflow);

                if (workflow is CompositeActivity)
                {
                    List<Activity> children = new List<Activity>();

                    GetChildActivities(workflow as CompositeActivity, children);

                    foreach (Activity child in children)
                    {
                        designer.DestroyComponent(child);
                    }
                }
            }
        }
    }
    

}
