using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using System.Collections.Generic;
using System.ComponentModel.Design;
using System.Workflow.ComponentModel.Design;
using System.Workflow.ComponentModel;
using System.Windows.Forms.Design;
using System.Linq;

namespace WorkflowSimulator
{
    /// <summary>
    /// A workflow menu service that provides a 
    /// context-menu for the selected component.
    /// </summary>
    public class WorkflowMenuService : MenuCommandService
    {
        public WorkflowMenuService(IServiceProvider provider)
            : base(provider)
        {
        }

        public override void ShowContextMenu(CommandID menuID, int x, int y)
        {
            base.ShowContextMenu(menuID, x, y);

            if (menuID == WorkflowMenuCommands.SelectionMenu)
            {
                ContextMenu contextMenu = new ContextMenu();

                WorkflowView workflowView = GetService(typeof(WorkflowView)) as WorkflowView;

                if (workflowView != null)
                {
                    contextMenu.Show(workflowView, workflowView.PointToClient(new Point(x, y)));
                }
            }
            else if (menuID == WorkflowMenuCommands.ZoomMenu)
            {
                ContextMenu contextMenu = new ContextMenu();

                WorkflowView workflowView = GetService(typeof(WorkflowView)) as WorkflowView;

                if (workflowView != null)
                {
                    MenuCommand mc = new MenuCommand(new EventHandler(
                        delegate(object sender, EventArgs e) { workflowView.FitToScreenSize(); })
                        , new CommandID(new Guid(), 11235));

                    MenuItem menuItem = new MenuItem("Fit To Screen", new EventHandler(OnMenuClicked));
                    menuItem.Tag = mc;
                    contextMenu.MenuItems.Add(menuItem);

                    menuItem = new MenuItem("100%", new EventHandler(OnMenuClicked));
                    menuItem.Tag = FindCommand(WorkflowMenuCommands.Zoom100Mode);
                    contextMenu.MenuItems.Add(menuItem);

                    menuItem = new MenuItem("50%", new EventHandler(OnMenuClicked));
                    menuItem.Tag = FindCommand(WorkflowMenuCommands.Zoom50Mode);
                    contextMenu.MenuItems.Add(menuItem);

                    menuItem = new MenuItem("-");
                    contextMenu.MenuItems.Add(menuItem);

                    menuItem = new MenuItem("Zoom In", new EventHandler(OnMenuClicked));
                    menuItem.Tag = FindCommand(WorkflowMenuCommands.ZoomIn);
                    contextMenu.MenuItems.Add(menuItem);

                    menuItem = new MenuItem("Zoom Out", new EventHandler(OnMenuClicked));
                    menuItem.Tag = FindCommand(WorkflowMenuCommands.ZoomOut);
                    contextMenu.MenuItems.Add(menuItem);

                    menuItem = new MenuItem("-");
                    contextMenu.MenuItems.Add(menuItem);

                    menuItem = new MenuItem("Pointer", new EventHandler(OnMenuClicked));
                    menuItem.Tag = FindCommand(MenuCommands.KeyCancel);
                    contextMenu.MenuItems.Add(menuItem);

                    contextMenu.Show(workflowView, workflowView.PointToClient(new Point(x, y)));

                }

            }
            else if (menuID == WorkflowMenuCommands.PanMenu)
            {
                FindCommand(WorkflowMenuCommands.Pan).Invoke();
            }
            else if (menuID == WorkflowMenuCommands.DesignerActionsMenu)
            {
                ContextMenu contextMenu = new ContextMenu();

                WorkflowView workflowView = GetService(typeof(WorkflowView)) as WorkflowView;

                foreach (var verb in this.GetCommandList(menuID.Guid).OfType<DesignerVerb>())
                {
                    MenuItem menuItem = new MenuItem(verb.Text, new EventHandler(OnMenuClicked));
                    menuItem.Tag = verb;
                    contextMenu.MenuItems.Add(menuItem);

                    contextMenu.Show(workflowView, workflowView.PointToClient(new Point(x, y)));
                }
            }
        }

        private void OnMenuClicked(Object sender, EventArgs e)
        {
            if (sender is MenuItem)
            {
                MenuItem menu = sender as MenuItem;
                if ((menu != null) && (menu.Tag is MenuCommand))
                {
                    ((MenuCommand)menu.Tag).Invoke();
                }
            }
        }
    }
}
